from ftrack_qt.widgets.frames.asset_version_creation import (
    AssetVersionCreation,
)
from ftrack_qt.widgets.frames.asset_version_selection import (
    AssetVersionSelection,
)
from ftrack_qt.widgets.frames.new_asset_input import NewAssetInput
